package eu.mhsl.minecraft.pixelblocks;

import eu.mhsl.minecraft.pixelblocks.commands.ChessCommand;
import eu.mhsl.minecraft.pixelblocks.commands.CreatePixelBlockCommand;
import eu.mhsl.minecraft.pixelblocks.commands.ExitWorldCommand;
import eu.mhsl.minecraft.pixelblocks.listeners.*;
import eu.mhsl.minecraft.pixelblocks.pixelblock.PixelBlock;
import org.bukkit.Bukkit;
import org.bukkit.Location;
import org.bukkit.plugin.java.JavaPlugin;

import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Objects;
import java.util.UUID;

public final class PixelBlocks extends JavaPlugin {
    public static PixelBlocks plugin;

    @Override
    public void onEnable() {
        PixelBlocks.plugin = this;
        this.getLogger().info("PixelBlocks Plugin was enabled.");
        this.loadDB();

        getServer().getPluginManager().registerEvents(new PlayerJumpListener(), this);
        getServer().getPluginManager().registerEvents(new PlayerInteractListener(), this);
        getServer().getPluginManager().registerEvents(new PlayerMoveListener(), this);
        getServer().getPluginManager().registerEvents(new EntityDamageListener(), this);
        getServer().getPluginManager().registerEvents(new BlockBreakListener(), this);
        getServer().getPluginManager().registerEvents(new BlockPlaceListener(), this);
        getServer().getPluginManager().registerEvents(new CreatureSpawnListener(), this);
        getServer().getPluginManager().registerEvents(new PlayerChangeWorldListener(), this);

        Objects.requireNonNull(getCommand("chess")).setExecutor(new ChessCommand());
        Objects.requireNonNull(getCommand("createpixelblock")).setExecutor(new CreatePixelBlockCommand());
        Objects.requireNonNull(getCommand("exitworld")).setExecutor(new ExitWorldCommand());
    }

    public void loadDB() {
        String url = "jdbc:sqlite:pixelblocks.db";
        String sql = "CREATE TABLE IF NOT EXISTS pixelblocks (" +
            "uuid CHAR(36) PRIMARY KEY, " +
            "owner CHAR(36), " +
            "locationWorldName CHAR(36), " +
            "locationX DOUBLE, " +
            "locationY DOUBLE, " +
            "locationZ DOUBLE" +
            ");";

        try (var conn = DriverManager.getConnection(url);
            var stmt = conn.createStatement()) {
            stmt.execute(sql);

            ResultSet pixelBlocksResult = stmt.executeQuery("SELECT * FROM pixelblocks;");
            while (pixelBlocksResult.next()) {

                Location newPixelBlockLocation = new Location(Bukkit.getWorld(pixelBlocksResult.getString("locationWorldName")),
                    pixelBlocksResult.getDouble("locationX"),
                    pixelBlocksResult.getDouble("locationY"),
                    pixelBlocksResult.getDouble("locationZ"));

                PixelBlock newPixelBlock = new PixelBlock(
                    newPixelBlockLocation,
                    UUID.fromString(pixelBlocksResult.getString("owner")),
                    16
                );
                newPixelBlock.remove();
                newPixelBlock.place(newPixelBlockLocation);
            }

        } catch (SQLException e) {
            System.err.println(e.getMessage());
        }
    }

}
